/** 
 *   Site testing for SKA candidate.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <ctype.h>
#include <stdio.h>
#include <termio.h>
#include <sys/file.h>

#include "my_stdio.h"


#define STR_ID      0xff
#define SET_AZ      0x80
#define SET_POL     0x81
#define SET_AZ_POL  0x82
#define SET_VEL     0x88
#define SET_CONTROL 0x87

#define GET_AZ      0x83
#define GET_POL     0x84
#define GET_STATUS  0x85
#define GET_RESET   0x86

#define POL_HOR  0
#define POl_VERT 1

/* for GET_STATUS: */
#define ST_MASK  0x7f
#define ST_CCW   0x01
#define ST_CW    0x02
//#define ST_      0x04
//#define ST_      0x08 
#define ST_INPOS 0x10
#define ST_CON_M 0x20
#define ST_ON    0x40

/* for GET_RESET: */
#define RS_MASK   0x0f
//#define RS_XXX    0x01 /* TODO */
#define RS_DC     0x02
#define RS_EERRO  0x04
#define RS_REMOTO 0x08

/* for SET_CONTROL: */
#define CT_MASK  0x73
#define CT_AZ_EN 0x01
#define CT_PO_EN 0x02
#define CT_RESET 0x10
#define CT_CONS  0x20
#define CT_PW_ON 0x40



int rotador_setup(char *name)
{
	int dev;
	struct termio term;
	
	/* Open Serial Port : 8 bits data, 1 bit stop, No parity, 2400
           baud */
	
	if ((dev = open(name, O_RDWR)) < 0 ){
		PVERB(PERR, "Cannot open serial port.\n");
		return -1;
	}
	    
	/* Zero out the baud rate portion of c_cflag */
	term.c_cflag &= ~CBAUD;
	
	/* Set baud rate to 1200 */
	term.c_cflag |= B1200;
	term.c_cflag &= ~(CSIZE|PARENB);
	term.c_cflag |= CS8 ;
  
	ioctl(dev, TCSETAW, &term);
	ioctl(dev, TCSETAW, &term);
  
	/* Turn of Canonical Processing and echo */
	term.c_lflag &= ~ICANON;
	term.c_lflag &= ~ECHO;
  
	/* Set MIN to one and TIME to zero */
	term.c_cc[VMIN]  = 1;
	term.c_cc[VTIME] = 0;
  
	/* Disable NL to CR mapping */
	term.c_iflag &= ~ICRNL;
  
	/* Set new terminal state */
	ioctl(dev, TCSETA, &term);
	ioctl(dev, TCSETA, &term);
	
	return dev;
}
	

int rotador_set_azimut(int fd, int az)
{
	char answer[4];
	
	fd_printf(fd,"%c%c%c%c", STR_ID, SET_AZ, (az >> 8) & 0x3f, az & 0xff);
	read(fd, answer, 4);

	if(answer[0] != (char) STR_ID ||
	   answer[1] != (char) SET_AZ   ) {
		return -1;
	}

	/* TODO: wait for confirmation... */

	return 0;
}

